
// (c) COPYRIGHT URI/MIT 1999
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors: James Gallagher

#include "config_ff.h"

static char rcsid[] not_used ={"$Id: FFFloat32.cc,v 1.3.4.1 2003/09/06 23:33:14 jimg Exp $"};

#ifdef __GNUG__
#pragma implementation
#endif

#include "FFFloat32.h"
#include "util_ff.h"
#include "util.h"

extern long BufPtr;
extern char *BufVal;

Float32 *
NewFloat32(const string &n)
{
    return new FFFloat32(n);
}

FFFloat32::FFFloat32(const string &n) : Float32(n)
{
}

BaseType *
FFFloat32::ptr_duplicate()
{
    return new FFFloat32(*this); // Copy ctor calls duplicate to do the work
}
 
/** Read a Float32.

    @note Does not use either the \e in_selection or \e send_p properties. If
    this method is called and the \e read_p property is not true, the values
    are read.

    @exception Error if the size of the returned data is zero.
    @param dataset The name of the data file. Must have a matching FreeForm
    format file.
    @return Always returns false. */
bool
FFFloat32::read(const string &dataset)
{
    if (read_p())		// nothing to do
	return false;
  
    if (BufVal) {		// data in cache
	char * ptr = BufVal+BufPtr;

	dods_float32 align;
	memcpy((void*)&align, (void *)ptr, width());

	val2buf((void *) &align);
	set_read_p(true);

	BufPtr += width();
    }

    return false;
}

// $Log: FFFloat32.cc,v $
// Revision 1.3.4.1  2003/09/06 23:33:14  jimg
// I modified the read() method implementations so that they test the new
// in_selection property. If it is true, the methods will read values
// even if the send_p property is not true. This is so that variables used
// in the selection part of the CE, or as function arguments, will be read.
// See bug 657.
//
// Revision 1.3  2003/02/10 23:01:52  jimg
// Merged with 3.2.5
//
// Revision 1.2.2.1  2002/12/18 23:30:42  pwest
// gcc3.2 compile corrections, mainly regarding the using statement
//
// Revision 1.2  2000/10/11 19:37:56  jimg
// Moved the CVS log entries to the end of files.
// Changed the definition of the read method to match the dap library.
// Added exception handling.
// Added exceptions to the read methods.
//
// Revision 1.1  1999/05/19 17:52:41  jimg
// Added
//
